import { Contract, ethers } from 'ethers';
import { networkConfigs } from './utils/wallet-helper';
import { handleError } from './utils/utils';
import { _abi } from './contracts/ERC721Offsetable';
import axios from 'axios';
export var MintStatus;
(function (MintStatus) {
  MintStatus[MintStatus["CLOSED"] = 0] = "CLOSED";
  MintStatus[MintStatus["PUBLIC"] = 1] = "PUBLIC";
  MintStatus[MintStatus["PRIVATE"] = 2] = "PRIVATE";
})(MintStatus || (MintStatus = {}));
export class Block {
  constructor(contractAddress, testnet, type, walletInfo) {
    this.contractAddress = contractAddress;
    this.testnet = testnet;
    this.size = 0;
    this.contract = {};
    this.walletInfo = walletInfo;
    this.mintPrice = '0';
    this.type = type;
    this.status = MintStatus.CLOSED;
    this.personalAllocationLeft = 0;
    this.tokensOwned = 0;
    this.isInitialised = false;
    this.message = {};
  }
  async init() {
    try {
      if (this.type === 'ERC721') {
        if (!this.contractAddress.match(/^0x[a-fA-F0-9]{40}$/)) {
          throw new Error('Missing or invalid contract address');
        }
        await this._checkChain();
        await this._getContractDetails();
        this.isInitialised = true;
        window.ethereum.on('disconnect', () => {
          console.log('Disconnected');
          window.location.reload();
        });
        window.ethereum.on('accountsChanged', () => {
          console.log('Tactical reload');
          window.location.reload();
        });
      }
    }
    catch (error) {
      console.log(error);
    }
  }
  static async create(contractAddress, testnet, type, walletInfo) {
    try {
      const block = new Block(contractAddress, testnet, type, walletInfo);
      await block.init();
      return block;
    }
    catch (error) {
      return null;
    }
  }
  async mint(quantity) {
    try {
      this.message = {};
      const amountInEth = ethers.utils.formatEther(this.mintPrice);
      const amount = ethers.utils.parseEther(amountInEth).mul(quantity);
      if (this.status === MintStatus.PUBLIC) {
        await this._publicMint(quantity, amount);
      }
      else if (this.status === MintStatus.PRIVATE) {
        await this._privateMint(quantity, amount);
      }
    }
    catch (error) {
      this.message = { error: error.toString() };
    }
  }
  async _publicMint(quantity, amount) {
    try {
      const estimate = await this.contract.estimateGas.publicMint(quantity, { value: amount });
      const safeGasLimit = Math.floor(parseInt(estimate.toString()) * 1.2);
      const tx = await this.contract.publicMint(quantity, { gasLimit: safeGasLimit, value: amount });
      this.message = { transaction: tx.hash };
    }
    catch (error) {
      handleError(error);
    }
  }
  async _privateMint(quantity, amount) {
    try {
      console.log(quantity);
      console.log(amount);
      console.log(this.walletSignature);
      const estimate = await this.contract.estimateGas.privateMint(quantity, this.walletSignature, { value: amount });
      const safeGasLimit = Math.floor(parseInt(estimate.toString()) * 1.2);
      const tx = await this.contract.privateMint(quantity, this.walletSignature, { gasLimit: safeGasLimit, value: amount });
      this.message = { transaction: tx.hash };
    }
    catch (error) {
      handleError(error);
    }
  }
  async _getContractDetails() {
    try {
      this.contract = new Contract(this.contractAddress, _abi, this.walletInfo.signer);
      if (!this.contract) {
        throw new Error('Contract initialization failed.');
      }
      const config = await this.contract.mintConfig();
      const [, mintPrice, maxPerAddress, status] = config;
      this.totalSupply = (await this.contract.totalSupply()).toNumber();
      this.maxSupply = (await this.contract.maxSupply()).toNumber();
      this.tokensOwned = (await this.contract.tokensOfOwner(this.walletInfo.signerAddress)).length;
      this.personalAllocationLeft = Math.max(0, maxPerAddress - this.tokensOwned);
      this.mintPrice = mintPrice.toString();
      this.maxPerAddress = maxPerAddress;
      this.status = status;
      if (this.status === MintStatus.PRIVATE) {
        await this.getWalletSignature();
      }
    }
    catch (error) {
      handleError(error);
    }
  }
  async getWalletSignature() {
    var _a;
    try {
      const response = await axios.post('https://struct--api.herokuapp.com/getWalletSignature', {
        contractAddress: this.contractAddress,
        wallet: this.walletInfo.signerAddress,
      }, {
        headers: {
          'Content-Type': 'application/json',
        },
      });
      const sig = response.data;
      if (sig.signature) {
        this.walletSignature = sig.signature;
        this.message = { info: 'Address whitelisted' };
      }
      else {
        this.message = { error: 'Address not whitelisted' };
      }
    }
    catch (e) {
      if (((_a = e === null || e === void 0 ? void 0 : e.response) === null || _a === void 0 ? void 0 : _a.status) === 400) {
        this.message = { error: 'Address not whitelisted' };
      }
    }
  }
  async _checkChain() {
    const requiredNetwork = this.testnet ? networkConfigs.rinkeby : networkConfigs.homestead;
    if (!((await this.walletInfo.provider.getNetwork()).name === requiredNetwork.name)) {
      try {
        await window.ethereum.request({
          method: 'wallet_switchEthereumChain',
          params: [{ chainId: requiredNetwork.chainId }], // chainId must be in hexadecimal numbers
        });
        window.location.reload();
      }
      catch (error) {
        handleError(error);
      }
    }
  }
}
